/****************************************************************************
 *
 *   (c) 2009-2016 QGROUNDCONTROL PROJECT <http://www.qgroundcontrol.org>
 *
 * QGroundControl is licensed according to the terms in the file
 * COPYING.md in the root of the source code directory.
 *
 ****************************************************************************/

import QtQuick          2.3
import QtQuick.Controls 1.2
import QtLocation       5.3
import QtPositioning    5.3

import QGroundControl               1.0
import QGroundControl.ScreenTools   1.0
import QGroundControl.Palette       1.0
import QGroundControl.Controls      1.0
import QGroundControl.FlightMap     1.0

Item {
    id: _root

    property var map        ///< Map control to place item in
    property var qgcView    ///< QGCView to use for popping dialogs
    property var areaItem   ///< GeoArea derived class.

    property var _polygon:          areaItem
    property var _depot: 			undefined
    property bool _showDepot: 		areaItem.interactive || areaItem.borderPolygon.interactive

    signal clicked(int sequenceNumber)

    on_ShowDepotChanged: {
        if (_showDepot){
            _addDepot()
        } else {
            _destroyDepot()
        }
    }

    Component.onCompleted: {
        if (_showDepot){
            _addDepot()
        }
    }

    Component.onDestruction: {
        _destroyDepot()
    }

    // Area polygon
    QGCMapPolygonVisuals {
        id:                 mapPolygonVisuals
        mapControl:         map
        mapPolygon:         areaItem
        interactive:        true
        borderWidth:        1
        borderColor:        "black"
        interiorColor:      "blue"
        altColor:           QGroundControl.globalPalette.surveyPolygonTerrainCollision
        interiorOpacity:    _root.opacity
    }

    // Depot Point.
    Component {
        id: depotPointComponent
        DragCoordinate {
            property var depot: _root.areaItem.depot

            map:        _root.map
            qgcView:    _root.qgcView
            z:          QGroundControl.zOrderMapItems
            checked:    _root._showDepot
            coordinate: _root.areaItem.depot
            label: 		"Depot"

            function syncAndBind(){
                if (coordinate.latitude !== depot.latitude ||
                        coordinate.longitude !== depot.longitude){
                    if (_root.areaItem.containsCoordinate(coordinate)){
                        _root.areaItem.depot = coordinate
                    }
                }
                coordinate = Qt.binding(function(){return _root.areaItem.depot})
            }

            onDragReleased: {
                syncAndBind()
            }

            Component.onCompleted: {
                syncAndBind()
            }

        }
    }

    function _addDepot() {
        if (!_depot){
            _depot = depotPointComponent.createObject(_root)
            map.addMapItem(_depot)
        }
    }

    function _destroyDepot() {
        if (_depot){
            map.removeMapItem(_depot)
            _depot.destroy()
            _depot = undefined
        }
    }
}
